'''
LDAPi.py

Copyright 2006 Andres Riancho

This file is part of w3af, w3af.sourceforge.net .

w3af is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

w3af is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with w3af; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

'''

import core.data.kb.vuln as vuln
from core.data.fuzzer.fuzzer import *
import core.controllers.outputManager as om
from core.controllers.basePlugin.baseAuditPlugin import baseAuditPlugin
import core.data.kb.knowledgeBase as kb
from core.controllers.w3afException import w3afException
import core.data.parsers.urlParser as urlParser

class LDAPi(baseAuditPlugin):
	'''
	This plugin tests for LDAP injection bugs.
	@author: Andres Riancho ( andres.riancho@gmail.com )
	'''

	def __init__(self):
		baseAuditPlugin.__init__(self)
		
	def _fuzzRequests(self, freq ):
		'''
		Tests an URL for LDAP injection vulnerabilities.
		
		@param freq: A fuzzableRequest
		'''
		om.out.debug( 'LDAPi plugin is testing: ' + freq.getURL() )
		
		ldapiStrings = self._getLDAPiStrings()
		mutants = createMutants( freq , ldapiStrings )
			
		for mutant in mutants:
			if self._hasNoBug( 'LDAPi','LDAPi',mutant.getURL() , mutant.getVar() ):
				# Only spawn a thread if the mutant has a modified variable
				# that has no reported bugs in the kb
				targs = (mutant,)
				self._tm.startFunction( target=self._sendMutant, args=targs , ownerObj=self )
		
			
	def _getLDAPiStrings( self ):
		'''
		Gets a list of strings to test against the web app.
		
		@return: A list with all LDAPi strings to test.
		'''
		LDAPstr = []
		LDAPstr.append("^(#$!@#$")
		LDAPstr.append(")(()))")
		return LDAPstr

	def _analyzeResult( self, mutant, response ):
		'''
		Analyze results of the _sendMutant method.
		'''
		if self._findLDAPError( response.getCode() , response.getBody() ):
			v = vuln.vuln( mutant )
			v.setId( response.id )
			v.setDesc( 'LDAP injection was found at: ' + response.getURL() + ' . Using method: ' + v.getMethod() + '. The data sent was: ' + str(mutant.getDc()) )
			kb.kb.append( self, 'LDAPi', v )
	
	def end(self):
		'''
		This method is called when the plugin wont be used anymore.
		'''
		self._tm.join( self )
		self.printUniq( kb.kb.getData( 'LDAPi', 'LDAPi' ), 'VAR' )
		
	def _findLDAPError( self, code, htmlString ):
		'''
		This method searches for LDAP errors in html's.
		
		@parameter code: The response code for the test.
		@parameter htmlString: The html string where the method searches for ldap errors
		@return: True if a LDAP was found on the site, False otherwise.
		'''
		# A lot of false positives, but this will remain like this until I get a bigger list
		# of ldap errors
		if code == 500:
			return True
		
		ldapErrors = self._getLDAPErrors()

		for ldapError in ldapErrors:
			position = htmlString.lower().find( ldapError )
			if  position != -1:
				om.out.vulnerability('Found Ldap injection. The error showed by the Web App is (fragment): "' + htmlString[ position - 20 : position + 20]  + '".')
				return True
				
		return False
		
	def _getLDAPErrors( self ):
		errorStr = []
		errorStr.append('supplied argument is not a valid ldap')
		errorStr.append('javax.naming.NameNotFoundException'.lower())
		# This one will generate some false positives, but i'll leve it here for now
		# until i have a complete list of errors.
		errorStr.append('ldap')
		return errorStr
		
	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		w3af/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
		</OptionList>\
		'

	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		pass

	def getPluginDeps( self ):
		'''
		@return: A list with the names of the plugins that should be runned before the
		current one.
		'''
		return []
	
	def getLongDesc( self ):
		'''
		@return: A DETAILED description of the plugin functions and features.
		'''
		return '''
		This plugin will find LDAP injections.
		'''
