'''
phishingVector.py

Copyright 2006 Andres Riancho

This file is part of w3af, w3af.sourceforge.net .

w3af is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

w3af is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with w3af; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

'''

from core.data.fuzzer.fuzzer import *
import core.controllers.outputManager as om
from core.controllers.basePlugin.baseAuditPlugin import baseAuditPlugin
import core.data.kb.knowledgeBase as kb
from core.controllers.w3afException import w3afException
import core.data.kb.vuln as vuln
import re

class phishingVector(baseAuditPlugin):
	'''
	This plugin searches for phishing vectors.
	
	@author: Andres Riancho ( andres.riancho@gmail.com )
	'''

	def __init__(self):
		baseAuditPlugin.__init__(self)
		self._urlForTesting = 'http://w3af.sf.net/'

	def _fuzzRequests(self, freq ):
		'''
		Find those phishing vectors!
		
		@param freq: A fuzzableRequest
		'''
		om.out.debug( 'phishingVector plugin is testing: ' + freq.getURL() )
		
		mutants = createMutants( freq , [self._urlForTesting] )
			
		for mutant in mutants:
			if self._hasNoBug( 'phishingVector' , 'phishingVector' , mutant.getURL() , mutant.getVar() ):
				# Only spawn a thread if the mutant has a modified variable
				# that has no reported bugs in the kb
				targs = (mutant,)
				self._tm.startFunction( target=self._sendMutant, args=targs, ownerObj=self )
		
			
	def _analyzeResult( self, mutant, response ):
		'''
		Analyze results of the _sendMutant method.
		'''
		response = self._findPhishingVector( mutant, response )
		for v in response:
			kb.kb.append( self, 'phishingVector', v )
	
	def _findPhishingVector( self, mutant, response ):
		'''
		Find the phishing vectors!
		'''
		res = []
		htmlBody = response.getBody()
		if htmlBody.find( self._urlForTesting ) != -1:
			# Houston we have a problem ;)
			frameRegex = re.compile('<(iframe|frame).*src=(\'|")?' + self._urlForTesting + '.*?>', re.DOTALL )
			if frameRegex.search( htmlBody ):
				# Vuln vuln!
				v = vuln.vuln( mutant )
				v.setId( response.id )
				v.setDesc( 'Phishing vector was found at: ' + v.getURL() + ' . Using method: ' + v.getMethod() + '. The data sent was: ' + str(v.getDc()) )
				res.append( v )
		return res
		
	
	def end(self):
		'''
		This method is called when the plugin wont be used anymore.
		'''
		self._tm.join( self )
		self.printUniq( kb.kb.getData( 'phishingVector', 'phishingVector' ), 'VAR' )

	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		w3af/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
		</OptionList>\
		'

	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		pass

	def getPluginDeps( self ):
		'''
		@return: A list with the names of the plugins that should be runned before the
		current one.
		'''
		return []
	
	def getLongDesc( self ):
		'''
		@return: A DETAILED description of the plugin functions and features.
		'''
		return '''
		This plugins finds phishing vectors in web applications, for example, a bug of this type is found
		if I request the URL "http://www.XXX.com/asd.asp?info=http://www.google.com" and in the response
		HTML the web application sends: 
		... 
		<iframe src="http://www.google.com">
		....
		'''
